import * as elements from '@yellicode/elements';
import * as opts from './options';
import { CodeWriter, TextWriter } from '@yellicode/templating';
export declare class CSharpWriter extends CodeWriter {
    private typeNameProvider;
    private commentWriter;
    /**
     * Constructor. Creates a new CSharpWriter instance using the TextWriter and options provided.
     * @param writer The template's current TextWriter.
     * @param options Optional: the global options for this writer.
     */
    constructor(writer: TextWriter, options?: opts.WriterOptions);
    /**
     * Writes 1 or more using directives, each on a new line.
     * @param values A collection of strings, typically namespace names.
     */
    writeUsingDirectives(...values: string[]): void;
    /**
     * Writes an indented block of code, wrapped in opening and closing brackets.
     * @param contents A callback function that writes the contents.
     */
    writeCodeBlock(contents: (writer: CSharpWriter) => void): void;
    /**
     * Writes an indented block of code, wrapped in a namespace declaration and opening and closing brackets.
     * @param pack A package that represents the namespace.
     * @param contents A callback function that writes the namespace contents.
     * @param options An optional NamespaceOptions object.
     */
    writeNamespaceBlock(pack: elements.Package, contents: (writer: CSharpWriter) => void, options?: opts.NamespaceOptions): void;
    writeNamespaceBlock(name: string, contents: (writer: CSharpWriter) => void): void;
    /**
     * Writes a block of code, wrapped in a class declaration and opening and closing brackets.
     * This function does not write class members.
     * @param cls The class.
     * @param contents A callback function that writes the class contents.
     * @param options An optional ClassOptions object.
     */
    writeClassBlock(cls: elements.Class, contents: (writer: CSharpWriter) => void, options?: opts.ClassOptions): void;
    /**
    * Writes a block of code, wrapped in an interface declaration and opening and closing brackets.
    * This function does not write interface members.
    * @param iface The interface.
    * @param contents A callback function that writes the interface contents.
    * @param options An optional InterfaceOptions object.
    */
    writeInterfaceBlock(iface: elements.Interface, contents: (writer: CSharpWriter) => void, options?: opts.InterfaceOptions): void;
    /**
    * Writes a block of code, wrapped in an enum declaration and opening and closing brackets.
    * This function does not write enumeration members. Use the writeEnumMember function
    * to write each individual member or the writeEnumeration function to write the full enumeration.
    * @param element The enumeration.
    * @param contents A callback function that writes the enumeration contents.
    * @param options An optional EnumerationOptions object.
    */
    writeEnumerationBlock(enumeration: elements.Enumeration, contents: (writer: CSharpWriter) => void, options?: opts.EnumOptions): void;
    /**
    * Writes a full enumeration, including members.
    * @param element The enumeration.
    * @param options An optional EnumOptions object.
    */
    writeEnumeration(enumeration: elements.Enumeration, options?: opts.EnumOptions): void;
    /**
     * Writes an individual enumeration member.
     * @param literal The EnumerationLiteral for which to write the member.
     * @param options An optional EnumMemberOptions object.
     */
    writeEnumMember(literal: elements.EnumerationLiteral, options?: opts.EnumMemberOptions): void;
    /**
    * Writes a method declaration without a body.
    * @param operation The operation for which to write the method.
    * @param options An optional MethodOptions object.
    */
    writeInterfaceMethod(operation: elements.Operation, options?: opts.MethodOptions): void;
    /**
    * Writes an indented block of code, wrapped in a method declaration and opening and closing brackets.
    * @param operation The operation for which to write the method.
    * @param contents A callback function that writes the operation contents.
    * @param options An optional MethodOptions object.
    */
    writeClassMethodBlock(operation: elements.Operation, contents: (writer: CSharpWriter) => void, options?: opts.MethodOptions): void;
    /**
     * Writes an auto property with a getter and (if the property is not ReadOnly) a setter.
     * This function can be used for both Classes and Interfaces.
     */
    writeAutoProperty(property: elements.Property, options?: opts.PropertyOptions): void;
    /**
     * Writes a property code block using optional callback functions for writing the getter and setter contents.
     */
    writePropertyBlock(property: elements.Property, getterContents: () => void | null, setterContents: () => void | null, options?: opts.PropertyOptions): void;
    private writePropertyStart(property, options);
    /**
     * Gets a string containing the return type of the method. If the return type cannot be determined,
     * the string 'void' is returned. If the method returns a collection, a collection type is returned
     * based on the provided options.
     * @param operation The operation.
     * @param options An optional MethodOptions object.
     */
    getMethodReturnType(operation: elements.Operation, options?: opts.MethodOptions): string;
    /**
     * Writes the input and output parameters (all parameters except the return parameter) of a method.
     * @param params A collection of parameters.
     * @param options An optional MethodOptions object.
     */
    writeInOutParameters(params: elements.Parameter[], options?: opts.MethodOptions): void;
    /**
    * Writes the visibility to the output with a trailing whitespace. If the visibilty is null or
    * not supported by C#, nothing will be written.
    * @param visibilityKind A VisibilityKind value. This value can be null.
    */
    writeAccessModifier(visibilityKind: elements.VisibilityKind | null): void;
    private writeGeneralizations(generalizations, additional);
    private writeInterfaceRealizations(realizations, additional, hasGeneralizations);
    private getTypeName(typedElement);
    writeXmlDocSummary(comments: elements.Comment[]): void;
    writeXmlDocSummary(text: string): void;
    writeXmlDocParameters(element: elements.Operation): void;
    writeXmlDocReturns(element: elements.Operation): void;
    writeXmlDocParagraph(text: string): void;
    writeXmlDocLines(lines: string[]): void;
    writeDelimitedCommentParagraph(text: string): void;
    writeDelimitedCommentLines(lines: string[]): void;
    private joinWrite<TItem>(collection, separator, getStringFunc);
    static getAccessModifierString(visibility: elements.VisibilityKind | null): string | null;
    static createCollectionType(typeName: string, collectionType?: opts.CollectionType): string;
}
